@@ -11,37 +11,40 @@ pub static BIG_TEST_VALIDATE: Lazy<NbtCompound> = Lazy::new(|| {
         bytes.push(((n * n * 255 + n * 7) % 100) as i8);
     }
 
-    compound! {
-        "nested compound test": {
-            "egg": {
-                "name": "Eggbert",
-                "value": 0.5f32
+    #[allow(clippy::excessive_precision)]
+    {
+        compound! {
+            "nested compound test": {
+                "egg": {
+                    "name": "Eggbert",
+                    "value": 0.5f32
+                },
+                "ham": {
+                    "name": "Hampus",
+                    "value": 0.75f32
+                }
             },
-            "ham": {
-                "name": "Hampus",
-                "value": 0.75f32
-            }
-        },
-        "intTest": 2147483647i32,
-        "byteTest": 127i8,
-        "stringTest": "HELLO WORLD THIS IS A TEST STRING \u{C5}\u{C4}\u{D6}!",
-        "listTest (long)": [11i64, 12, 13, 14, 15],
-        "doubleTest": 0.49312871321823148f64,
-        "floatTest": 0.49823147058486938f32,
-        "longTest": 9223372036854775807i64,
-        "listTest (compound)": [
-            {
-                "created-on": 1264099775885i64,
-                "name": "Compound tag #0"
-            },
-            {
-                "created-on": 1264099775885i64,
-                "name": "Compound tag #1"
-            }
-        ],
-        "byteArrayTest (the first 1000 values of (n*n*255+n*7)%100, starting with n=0 (0, 62, 34, \
-            16, 8, ...))": bytes,
-        "shortTest": 32767i16
+            "intTest": 2147483647i32,
+            "byteTest": 127i8,
+            "stringTest": "HELLO WORLD THIS IS A TEST STRING \u{C5}\u{C4}\u{D6}!",
+            "listTest (long)": [11i64, 12, 13, 14, 15],
+            "doubleTest": 0.49312871321823148f64,
+            "floatTest": 0.49823147058486938f32,
+            "longTest": 9223372036854775807i64,
+            "listTest (compound)": [
+                {
+                    "created-on": 1264099775885i64,
+                    "name": "Compound tag #0"
+                },
+                {
+                    "created-on": 1264099775885i64,
+                    "name": "Compound tag #1"
+                }
+            ],
+            "byteArrayTest (the first 1000 values of (n*n*255+n*7)%100, starting with n=0 (0, 62, 34, \
+                16, 8, ...))": bytes,
+            "shortTest": 32767i16
+        }
     }
 });
 
@@ -100,10 +103,38 @@ pub static SNBT_EDGE_CASES_VALIDATE: Lazy<NbtCompound> = Lazy::new(|| {
         "quoted \"key\"": "quoted 'value'",
         "redundant": "quotes",
         "more_redundant": "quotes",
-        "escape sequences": "\'\\\r\n\t\u{00A7}\u{0F63}",
+        "escape sequences": "\'a\\bc\rd\nef\t\u{00A7}_\u{0F63}",
     }
 });
 
 pub const BIG_SNBT: &str = include_str!("big_snbt.snbt");
 
 pub const LEVEL_DAT: &[u8] = include_bytes!("level.dat");
+
+#[macro_export]
+macro_rules! assert_compound_eq {
+    ($test:expr, $validate:expr) => {{
+        let test = $test;
+        let validate = $validate;
+
+        for (key, value) in validate.inner().iter() {
+            let test_value = match test.inner().get(key) {
+                Some(test_value) => test_value,
+                None => {
+                    panic!(
+                        "Compound missing field {} found in \
+                         validation:\nTest:{:#?}\nValidate:{:#?}",
+                        key, test, validate
+                    );
+                }
+            };
+
+            if test_value != value {
+                panic!(
+                    "Test field does not equal validation field:\nTest:{:#?}\nValidate:{:#?}",
+                    test_value, value
+                )
+            }
+        }
+    }};
+}
